;*****************************************************************************
;*****************************************************************************
;** Projekt:     Optorzkel                                               **
;** Lers:      Behatols rzkel. A bal oldali LED soron alapesetben a   **
;** 		 zld LED vilgt. Ha valaki elhalad az opto rzkel eltt,**
;**		 akkor gyulladjanak ki a piros LED-ek s 2Hz-es		    **
;**		 frekvencival kezdjenek el villogni. Egszen addig 	    **
;**		 villogjanak, amg a kezelszemlyzet az INT megnyomsval  **
;**		 nem nyugtzta, hogy szrevettk a jelzst. Az idzts     **
;**		 hardveridztvel trtnjen. A nyomgomb 		    **
;**		 pergsmentestsvel nem kell foglalkozni! (15/16)	    **
;** Szerzo:      Major Pter			                            **
;** Mrocsoport: i1hdu13                                                    **
;** Kszlt:     2008. mrcius 1.  					    **
;*****************************************************************************
;*****************************************************************************
;
;*****************************************************************************
;* AVR mropanel portkioszts:
;*****************************************************************************
;*
;* LED0(P):PortC.0          LED4(P):PortC.4
;* LED1(P):PortC.1          LED5(P):PortC.5
;* LED2(S):PortC.2          LED6(S):PortC.6
;* LED3(Z):PortC.3          LED7(Z):PortC.7        INT:PortE.4
;*
;* SW0:PortG.0     SW1:PortG.1     SW2:PortG.4     SW3:PortG.3
;* 
;* BT0:PortE.5     BT1:PortE.6     BT2:PortE.7     BT3:PortB.7
;*
;*****************************************************************************
;*
;* AIN:PortF.0     NTK:PortF.1    OPTO:PortF.2     POT:PortF.3
;*
;*****************************************************************************
;*
;* LCD1(VSS) = GND         LCD9(DB2): -
;* LCD2(VDD) = VCC         LCD10(DB3): -
;* LCD3(VO ) = GND         LCD11(DB4): PortA.4
;* LCD4(RS ) = PortA.0     LCD12(DB5): PortA.5
;* LCD5(R/W) = GND         LCD13(DB6): PortA.6
;* LCD6(E  ) = PortA.1     LCD14(DB7): PortA.7
;* LCD7(DB0) = -           LCD15(BLA): VCC
;* LCD8(DB1) = -           LCD16(BLK): PortB.5 (1=Httrvilgts ON)
;*
;*****************************************************************************

.nolist
.include "m128def.inc"	; ATMega 128 defincis fjl 
.list

;***** Konstansok *****

.equ	tconst	= 25			; idoztsi konstans (T = 25*10 msec)
.equ 	brand = 100			; A fnyrzkel kszbszintje (max 255)

;***** Regiszterkioszts ***** 

.def	temp = r16			; lt. segdregiszter 
.def	temp2 =r17			; msodik regiszter, hogy el tudjuk trolni az A/D converter msik bitcsoportjt.
.def 	state = r18			; az llapot trolshoz. 1 ha van betr, 0 ha nem
.def	led	= r19			; itt troljuk el, hogy melyik led fog vilgtani, vagy villogni (ezt klldjk ki a PORTC-re)
.def	betoro = r20			; (1) ha van betr, (0) ha nincsen
.def	abort = r21			; itt troljuk el, hogy megnyomta-e valaki az INT gombot
.def	cyc = r22			;szoftveres ciklushoz itertor
;*****************************************************************************
;* Reset & IT vektorok  
;*****************************************************************************
		
.cseg 
		
.org	0x0000		; Kdszegmens kezdocme 
	
		jmp		main		; Reset vektor 
		jmp		dummy		; EXTINT0 Handler
		jmp		dummy		; EXTINT1 Handler
		jmp		dummy		; EXTINT2 Handler
		jmp		dummy		; EXTINT3 Handler
		jmp		pressed		; EXTINT4 Handler (INT gomb)
		jmp		dummy		; EXTINT5 Handler
		jmp		dummy		; EXTINT6 Handler
		jmp		dummy		; EXTINT7 Handler
		jmp		dummy		; Timer2 Compare Match Handler 
		jmp		dummy		; Timer2 Overflow Handler 
		jmp		dummy		; Timer1 Capture Event Handler 
		jmp		dummy		; Timer1 Compare Match A Handler 
		jmp		dummy		; Timer1 Compare Match B Handler 
		jmp		dummy		; Timer1 Overflow Handler 
		jmp		timeout		; Timer0 Compare Match Handler 
		jmp		dummy		; Timer0 Overflow Handler 
		jmp		dummy		; SPI Transfer Complete Handler 
		jmp		dummy		; USART0 RX Complete Handler 
		jmp		dummy		; USART0 Data Register Empty Hanlder 
		jmp		dummy		; USART0 TX Complete Handler 
		jmp		detect		; ADC Conversion Complete Handler 
		jmp		dummy		; EEPROM Ready Hanlder 
		jmp		dummy		; Analog Comparator Handler 
		jmp		dummy		; Timer1 Compare Match C Handler 
		jmp		dummy		; Timer3 Capture Event Handler 
		jmp		dummy		; Timer3 Compare Match A Handler 
		jmp		dummy		; Timer3 Compare Match B Handler 
		jmp		dummy		; Timer3 Compare Match C Handler 
		jmp		dummy		; Timer3 Overflow Handler 
		jmp		dummy		; USART1 RX Complete Handler 
		jmp		dummy		; USART1 Data Register Empty Hanlder 
		jmp		dummy		; USART1 TX Complete Handler 
		jmp		dummy		; Two-wire Serial Interface Handler 
		jmp		dummy		; Store Program Memory Ready Handler 
	
;*****************************************************************************
;* Fprogram
;*****************************************************************************
		
		.org	0x0046

;***** Stack inicializlsa *****

main:		ldi		temp, LOW(RAMEND)	; RAMEND = RAM vgcme
		out		SPL, temp		; (ld."m128def.inc") 
		ldi		temp, HIGH(RAMEND)
		out		SPH, temp

;***** Portok inicializlsa *****

	;*** PORTC.0-7: LED0-7 ***

		ldi		temp, 0xFF		; portbitek kimenetek
		out		DDRC, temp		; PORTC kimenet
		ldi 	led, 0b00001000			; alapbl a bal oldali zld led vilgt
		out 	PORTC, led			; kirjuk a led-ekre az rtket

	;*** PORTE.0-7: Gombok bemenetre lltsa ***

		ldi		temp, 0x00
		out 	DDRE, temp

;***** Az Interruptok kezelst bekapcsoljuk az INT gombra *****

		ldi 	temp, 0x02			; az INT gomb lefut lre reagljon
		out 	EICRB, temp
		ldi 	temp, 0b00010000
		out 	EIMSK, temp

;***** Az ADConverter inicializlsa

		ldi 	temp, 0b11100010		; A/D konverter belltsa: 2.56V feszltsg, balra rendezve adja az rtkeket, PORTF2-rl olvasunk
		out 	ADMUX, temp
		ldi 	temp, 0b11101111		; A/D konvertel engedlyezs, szabad futs md, interrupt egedlyezse, 128-as leoszts
		out 	ADCSRA, temp
		sbi 	ADCSRA, ADIE			; A/D converter beindtsa

;***** Vltozk inicializlsa *****
		ldi state, 0				; alapbl a zld llapotban vagyunk
		ldi cyc, 50				; a szoftverers ciklushoz vltoz				

;***** Timer 0 inicializlsa *****

		ldi		temp,0b00001111
				;	   0.......	; FOC=0
				;	   .0..1...	; WGM=10 (CTC mod)
				;	   ..00....	; COM=00 (kimenet tiltva)
				;	   .....111	; CS0=111 (CLK/1024)
		out		TCCR0,temp		; Timer 0 TCCR0 regiszter
		ldi		temp,216		; 11059200Hz/1024 = 108*100
		out		OCR0,temp		; Timer 0 OCR0 regiszter
		ldi		temp,0b00000010
				;	   000000..	; Timer2,1 IT tiltva
				;	   ......1.	; OCIE0=1
				;	   .......0	; TOIE0=0
		out		TIMSK,temp		; Timer IT Mask regiszter
		sei					; globlis IT engedlyezve

;*****************************************************************************
;vltozk inicializlsa itt trtnik:
loop:	; itt kezeljk az egyes llapotokat.
		cpi state, 0		; if (betoro == 0)
		breq state0		; then jump state0
		jmp state1		; else jump state1

state0:	
		cpi betoro, 0		; ha nincsen betr
		breq loop_end		; ksz

		; ha van betr
		ldi led, 0b00110011
		out PORTC, led		; belltjuk a led-eket
		ldi state, 1
		jmp loop_end

state1:
		cpi abort, 0		; ha nem volt megszakts -> ksz
		breq loop_end

		;ha volt
		clr abort
		ldi temp, 0x08
		out PORTC, temp		; csak a zld ledek vilgtanak
		ldi state, 0 		; tvltunk zld llapotba

loop_end:
		jmp		loop	; vgtelen hurok

;*************************************************************** 
;* Subroutins, Interrupt routins

pressed:				; az INT gomb megnyomst kezel interrupt
		ldi abort, 1		; belltjuk a megszaktst
		reti

detect:		
	push temp2			; segdregiszter mentse
	push temp		
	in temp, SREG			; sttuszregiszter mentse
	push temp

	ldi betoro, 0			; default rtk: nincs betr
	in temp, ADCH			; a magas bitek mentse

	cpi temp, brand			; sszehasonlts a kszbszinttel
	brcs detect_end 		; ha lesz carry, akkor 0-val trnk vissza (if (temp < brand))

detect_end_1:
	ldi betoro, 1			; egybknt 1-gyel
    	
detect_end:
	pop temp			; Interrupt eltti llapot visszatltse
	out SREG, temp
	pop temp
	pop temp2

	reti
;*****************************************************************************
;* 10 msec Timer IT rutin
;*****************************************************************************


	.dseg
	count:	.byte	tconst		; Timer szmll, helyfoglals RAM-ban 
	.cseg

; az idtl fgg essemnyek kezelse ---> a Piros led-ek villogtatsa, ha szksges
timeout:	push	temp					; segdregiszter mentse
			in temp,SREG				; sttusz mentse
			push	temp

			cpi state, 0				; ha zld llapotban vagyunk
			breq timeback				; akkor nem csinlunk semmit

			; ha piros llapotban vagyunk
			dec cyc					; szoftveres idzts a hardveres mell
			brne timeback				; ha nem vgzett, ne csinljon semmit			
			ldi cyc, 50				; ha vgzett, lltsa vissza az itertort, majd tegye a dolgt
			in temp, PINC				; betltjk a led-ek llst
			com temp 				; negljuk a ledek rtkeit
			andi temp, 0b00110011 			; kimaszkoljuk a piros ledeket
			out PORTC, temp 			; kirjuk

timeback:
			pop		temp			; regiszterek visszalltsa
			out		SREG,temp
			pop		temp
dummy:	reti
